import { Component, OnInit, ChangeDetectorRef } from '@angular/core';
import { Helpers } from '../shared/helpers.service';
import { Saml } from '../shared/saml';
import { Pair } from '../shared/pair';
import { AliasClaimRec } from '../shared/alias-claim-rec';
import { Info } from '../shared/Info';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.css']
})
export class AppComponent implements OnInit {
  title = 'XpAngular';
  hosted = 'false';
  currentUser = null;
  aliasClaim: jackHenry.enterprise.businessObjects.AliasClaimRec_CType;

  constructor() {
    this.hosted = jackHenry.xperience.isFrameworkHosted.toString();
  }
  ngOnInit(): void {
    jackHenry.xperience.messageBus.onReady(onFrameworkReady, 5000);
  }

  clearTextArea(): void {
    const textArea = document.getElementById('t-area');
    textArea.innerText = '';
  }

  alert(): void {
    window.alert(
      'Alerts are NOT good. They demand immediate attention and prevent other Xperience tabs from being selected.'
    );
  }

  addAlias(): void {
    const request = new jackHenry.enterprise.businessObjects.AliasAdd();
    request.AliasClaimRec = new jackHenry.enterprise.businessObjects.AliasClaimRec_CType();
    request.AliasClaimRec.Alias = 'Loans';
    request.AliasClaimRec.AliasContext = Helpers.guid();
    request.AliasClaimRec.AliasDesc = 'Loan Division';
    request.AliasClaimRec.AliasDft = 'false';
    request.AliasClaimRec.AliasInstRtId = '12345678';
    request.AliasClaimRec.AppliesToApp = 'Whatever';
    request.AliasClaimRec.UniqueId = Helpers.guid();
    request.IMSSubj = imsSubj;
    this.aliasClaim = request.AliasClaimRec;
    const responseCallback = jackHenry.xperience.messageBus
      .send(request)
      .registerResponseHandler(onAddAliasCallback);

    if (responseCallback.receipt.errors !== undefined) {
      const errors = Helpers.callbackErrors(responseCallback.receipt.errors);
      displayOutput(errors, 'addAlias');
    }

    if (!responseCallback.receipt.isValid()) {
      const error = Helpers.errorsToString(responseCallback.receipt.errors);
      displayOutput(error, 'addAlias');
    }
  }

  deleteAlias(): void {
    if (
      this.aliasClaim.UniqueId !== null &&
      this.aliasClaim.UniqueId.length > 0
    ) {
      const request = new jackHenry.enterprise.businessObjects.AliasMod();
      request.AliasClaimRec = this.aliasClaim;
      request.Dlt = 'true';
      request.IMSSubj = imsSubj;
      const responseCallback = jackHenry.xperience.messageBus
        .send(request)
        .registerResponseHandler(onAliasModCallback);

      if (responseCallback.receipt.errors !== undefined) {
        const errors = Helpers.callbackErrors(responseCallback.receipt.errors);
        displayOutput(errors, 'deleteAlias');
      }

      if (!responseCallback.receipt.isValid()) {
        const error = Helpers.errorsToString(responseCallback.receipt.errors);
        displayOutput(error, 'deleteAlias');
      }
    }
  }

  getKerberosToken(): void {
    const request = new jackHenry.enterprise.businessObjects.GetKerberosToken();

    // Service Principal Name (SvcPrincName) is *required* in the request.
    // TokenTimeoutMin is optional.
    //    Defaults to 60 minutes if omitted.
    //    Must be > zero if supplied.
    request.SvcPrincName = 'testSvc/jxtest.local';
    request.TokenTimeoutMin = 30;

    Helpers.fillHeader(request);
    const responseCallback = jackHenry.xperience.messageBus
      .send(request)
      .registerResponseHandler(onKerberosTokenCallback);
    if (!responseCallback.receipt.isValid()) {
      const error = Helpers.errorsToString(responseCallback.receipt.errors);
      displayOutput(error, 'getKerberosToken');
    } else {
      displayOutput(responseCallback.receipt.messageId, 'getKerberosToken');
    }
  }
}

let imsSubj: string;

// Make an object a string that evaluates to an equivalent object
//  Note that eval() seems tricky and sometimes you have to do
//  something like eval("a = " + yourString), then use the value
//  of a.
//
//  Also this leaves extra commas after everything, but JavaScript
//  ignores them.
function convertToText(obj) {
  // create an array that will later be joined into a string.
  const str = [];

  // is object
  //    Both arrays and objects seem to return "object"
  //    when typeof(obj) is applied to them. So instead
  //    I am checking to see if they have the property
  //    join, which normal objects don't have but
  //    arrays do.
  if (typeof obj === 'object' && obj.join === undefined) {
    str.push('{');
    for (const prop in obj) {
      if (obj.hasOwnProperty(prop)) {
        str.push(prop, ': ', convertToText(obj[prop]), ',');
      }
    }
    str.push('}');

    // is array
  } else if (typeof obj === 'object' && !(obj.join === undefined)) {
    str.push('[');
    for (const prop in obj) {
      if (obj.hasOwnProperty(prop)) {
        str.push(convertToText(obj[prop]), ',');
      }
    }
    str.push(']');

    // is function
  } else if (typeof obj === 'function') {
    str.push(obj.toString());

    // all other values can be done with JSON.stringify
  } else {
    str.push(JSON.stringify(obj));
  }

  return str.join('');
}

function onFrameworkReady() {
  this.hosted = jackHenry.xperience.isFrameworkHosted.toString();

  // This will be called when the framework is ready, or after a timeout.
  // Make sure we are hosted before doing any connections:
  if (jackHenry.xperience.isFrameworkHosted) {
    // NOTE: It is a good idea to subscribe to messages before you connect to
    // the message bus but subscribing can be done at any time
    jackHenry.xperience.messageBus.subscribe('AppCls', onAppCls);

    // Must connect with the message bus before sending any messages
    jackHenry.xperience.messageBus.connect();
    if (jackHenry.xperience.messageBus.isConnected) {
      // Framework is ready.
      // dump startup args
      argDump(jackHenry.xperience.startupArguments);

      // retrieve the user obj (SSO token)
      // ***** Please note that this is the proper way to retrieve the user token. *****
      // ***** DO NOT USE THE UserInq MESSAGE SINCE IT DOES NOT ISSUED BY AN *****
      // ***** APPROVED STS OR SIGNED BY A TRUSTED ISSUER *****
      const currentUserRequest = new jackHenry.enterprise.businessObjects.UsrSecTokenRetrv();

      // this ensures you have an updated token from the server for Aliases.
      currentUserRequest.RefreshFromServer = 'true';

      Helpers.fillHeader(currentUserRequest);

      // get user info
      const responseCallback = jackHenry.xperience.messageBus
        .send(currentUserRequest)
        .registerResponseHandler(onGetCurrentUserCallback);

      if (responseCallback.receipt.errors !== undefined) {
        const errors = Helpers.callbackErrors(responseCallback.receipt.errors);
        displayOutput(errors, 'onFrameworkReady');
      }

      const request = new jackHenry.enterprise.businessObjects.SiteInfoInq();

      Helpers.fillHeader(request);
      const siteCallback = jackHenry.xperience.messageBus
        .send(request)
        .registerResponseHandler(onSiteInfoCallback);
      if (!siteCallback.receipt.isValid()) {
        const error = Helpers.errorsToString(siteCallback.receipt.errors);
        displayOutput(error, 'getSiteInfo - receipt error');
      } else {
        displayOutput(
          siteCallback.receipt.messageId,
          'getSiteInfo:receipt Id'
        );
      }
    }
  }
}

function onAppCls(context: any) {
  const textArea = document.getElementById('t-area');
  textArea.innerText = '';

  displayOutput('AppCls received from XPERIENCE', 'onAppCls');

  try {
    const response = new jackHenry.enterprise.businessObjects.AppClsResponse();
    const closeStatus = 'ClsAck';

    response.ClsStat = closeStatus;
    response.RsStat = 'Success';

    context.reply(response);
    jackHenry.xperience.messageBus.disconnect(true);
  } catch (error) {
    console.log('AppCls lost the message bus');
    console.log(error);
  }

  window.close();
}

function argDump(args) {
  const table = document.getElementById('args');

  for (const key in args) {
    if (args.hasOwnProperty(key)) {
      const tr = document.createElement('tr');
      const td1 = document.createElement('td');
      const td2 = document.createElement('td');

      td1.style.fontWeight = '400';
      td1.style.backgroundColor = '#e6e3d9';
      td1.style.width = '25%';
      td1.style.whiteSpace = 'nowrap';
      td1.style.overflow = 'hidden';
      td1.style.textOverflow = 'ellipsis';
      td1.style.padding = '10px 7px';
      td1.style.verticalAlign = 'top';
      td2.style.borderLeft = '1px solid #777';
      td2.style.fontWeight = '500';
      td2.style.width = '75%';
      td2.style.wordBreak = 'break-all';
      td2.style.padding = '10px 7px';
      td2.style.verticalAlign = 'top';

      td1.innerText = key;
      td2.innerText = args[key];

      tr.appendChild(td1);
      tr.appendChild(td2);
      tr.style.borderBottom = '1px solid #777';
      table.appendChild(tr);
    } else if (args.hasOwnProperty(key)) {
      displayOutput(args[key], 'argDump');
    }
  }
}

function currentUserDump(user) {
  const table = document.getElementById('user');

  for (const key in user) {
    if (user.hasOwnProperty(key) && key !== 'samlAssertion') {
      if (key === 'IMSSubj') {
        imsSubj = user[key];
      }
      const tr = document.createElement('tr');
      const td1 = document.createElement('td');
      const td2 = document.createElement('td');

      td1.style.fontWeight = '400';
      td1.style.backgroundColor = '#e6e3d9';
      td1.style.width = '25%';
      td1.style.whiteSpace = 'nowrap';
      td1.style.overflow = 'hidden';
      td1.style.textOverflow = 'ellipsis';
      td1.style.padding = '10px 7px';
      td1.style.verticalAlign = 'top';
      td2.style.borderLeft = '1px solid #777';
      td2.style.fontWeight = '500';
      td2.style.width = '75%';
      td2.style.wordBreak = 'break-all';
      td2.style.padding = '10px 7px';
      td2.style.verticalAlign = 'top';

      td1.innerText = key;
      td2.innerText = user[key];

      tr.appendChild(td1);
      tr.appendChild(td2);
      tr.style.borderBottom = '1px solid #777';
      table.appendChild(tr);
    } else if (user.hasOwnProperty(key)) {
      displayOutput(user[key], 'currentUserDump');
    }
  }
}

function attributes(saml) {
  const x2js = new X2JS();
  const j = x2js.xml_str2json(saml) as Saml;
  if (j.Assertion.AttributeStatement) {
    const arr = JSON.stringify(j.Assertion.AttributeStatement);
    const shorter = '[' + arr.slice(14, arr.length - 1);
    const at = JSON.parse(shorter);
    return at as Pair[];
  }

  return null;
}

function roleDump(attributeArray: Pair[]) {
  const table = document.getElementById('roles');
  for (let i = 0; i < attributeArray.length; i++) {
    const element = attributeArray[i] as Pair;
    if (
      element._Name ===
      'http://schemas.microsoft.com/ws/2008/06/identity/claims/role'
    ) {
      let none = true;
      for (let index = 0; index < element.AttributeValue.length; index++) {
        const role = element.AttributeValue[index];
        if (role !== '') {
          const tr = document.createElement('tr');
          const td = document.createElement('td');
          td.innerText = role;
          td.style.borderLeft = '1px solid #777';
          td.style.fontWeight = '500';
          td.style.width = '75%';
          td.style.wordBreak = 'break-all';
          td.style.padding = '10px 7px';
          td.style.verticalAlign = 'top';

          tr.appendChild(td);
          tr.style.borderBottom = '1px solid #777';
          table.appendChild(tr);
          none = false;
        }
      }

      if (none) {
        const tr = document.createElement('tr');
        const td = document.createElement('td');

        td.innerText = 'no array';

        tr.appendChild(td);
        tr.style.borderBottom = '1px solid #777';
        table.appendChild(tr);
      }
    }
  }
}

function aliasDump(attributeArray: Pair[]) {
  const table = document.getElementById('aliases');
  for (let i = 0; i < attributeArray.length; i++) {
    const element = attributeArray[i] as Pair;
    if (
      element._Name ===
      'http://schemas.jackhenry.com/ims/2010/identity/claims/alias'
    ) {
      if (element.AttributeValue.length > 0) {
        for (let index = 0; index < element.AttributeValue.length; index++) {
          const alias = element.AttributeValue[index];
          if (alias !== '') {
            const aliasClaimRec = decodeURIComponent(alias);
            const x2js = new X2JS();
            const claim = x2js.xml_str2json(aliasClaimRec) as AliasClaimRec;
            const record = claim['AliasClaimRec'];

            table.appendChild(tableRow('Alias', record.Alias));
            table.appendChild(tableRow('AliasContext', record.AliasContext));
            table.appendChild(tableRow('AliasDesc', record.AliasDesc));
            table.appendChild(tableRow('AliasDft', record.AliasDft));
            table.appendChild(tableRow('AliasInstRtId', record.AliasInstRtId));
            table.appendChild(tableRow('AppliesToApp', record.AppliesToApp));
            table.appendChild(tableRow('UniqueId', record.UniqueId));
          } else {
            const tr = document.createElement('tr');
            const td = document.createElement('td');

            td.innerText = 'no alias';

            tr.appendChild(td);
            tr.style.borderBottom = '1px solid #777';
            table.appendChild(tr);
          }
        }
      } else {
        const tr = document.createElement('tr');
        const td1 = document.createElement('td');
        const td2 = document.createElement('td');

        td2.innerText = 'No Aliases found';

        tr.appendChild(td1);
        tr.appendChild(td2);
        tr.style.borderBottom = '1px solid #777';
        table.appendChild(tr);
      }
    }
  }
}

function tableRow(key: string, value: string) {
  const tr = document.createElement('tr');
  const td1 = document.createElement('td');
  const td2 = document.createElement('td');

  td1.style.fontWeight = '400';
  td1.style.backgroundColor = '#e6e3d9';
  td1.style.width = '25%';
  td1.style.whiteSpace = 'nowrap';
  td1.style.overflow = 'hidden';
  td1.style.textOverflow = 'ellipsis';
  td1.style.padding = '10px 7px';
  td1.style.verticalAlign = 'top';
  td2.style.borderLeft = '1px solid #777';
  td2.style.fontWeight = '500';
  td2.style.width = '75%';
  td2.style.wordBreak = 'break-all';
  td2.style.padding = '10px 7px';
  td2.style.verticalAlign = 'top';

  td1.innerText = key;
  td2.innerText = value;

  tr.appendChild(td1);
  tr.appendChild(td2);
  if (key === 'UniqueId') {
    tr.style.borderBottom = '5px solid #777';
  } else {
    tr.style.borderBottom = '1px solid #777';
  }

  return tr;
}

function policyDump(attributeArray: Pair[]) {
  const policyTable = document.getElementById('policies');
  for (let i = 0; i < attributeArray.length; i++) {
    const element = attributeArray[i] as Pair;
    if (
      element._Name ===
      'http://schemas.jackhenry.com/ims/2010/policy/claims/imsdomain'
    ) {
      const claim = element.AttributeValue;
      const tr = document.createElement('tr');
      const td1 = document.createElement('td');

      td1.style.fontWeight = '400';
      td1.style.backgroundColor = '#e6e3d9';
      td1.style.width = '25%';
      td1.style.whiteSpace = 'nowrap';
      td1.style.overflow = 'hidden';
      td1.style.textOverflow = 'ellipsis';
      td1.style.padding = '10px 7px';
      td1.style.verticalAlign = 'top';

      td1.innerText = claim;

      tr.appendChild(td1);
      tr.style.borderBottom = '5px solid #777';

      if (claim !== '') {
        policyTable.appendChild(tr);
      }
    }
  }
}

function onGetCurrentUserCallback(context: jackHenry.xperience.MessageContext) {
  const response = context.messageToJson<
    jackHenry.enterprise.businessObjects.UsrSecTokenRetrvResponse
  >();

  if (!context.isValid()) {
    const error = Helpers.errorsToString(context.messageErrors);
    displayOutput(error, 'onGetCurrentUserCallback');
  } else {
    this.currentUser = response.UsrSecTokenRec;

    currentUserDump(this.currentUser);
    const at = attributes(this.currentUser.samlAssertion);
    roleDump(at);
    aliasDump(at);
    policyDump(at);
  }
}

function refreshCurrentUser() {
  // Note: setting this to true will get a fresh token from IMS server for the current user
  const currentUserRequest = new jackHenry.enterprise.businessObjects.UsrSecTokenRetrv();
  currentUserRequest.RefreshFromServer = 'true';

  jackHenry.xperience.messageBus
    .send(currentUserRequest)
    .registerResponseHandler(onGetCurrentUserCallback);
}

function displayOutput(text: string, func: string, clear: boolean = false) {
  if (clear) {
    console.log(func + ' - ' + text);
    const textArea = document.getElementById('t-area');
    textArea.innerText = text;
  } else {
    console.log(func + ' - ' + text);
    const textArea = document.getElementById('t-area');
    const current = textArea.innerText;
    textArea.innerText = current + '\n' + text;
  }
}

function onAddAliasCallback(context: jackHenry.xperience.MessageContext) {
  if (!context.isValid()) {
    const error = Helpers.errorsToString(context.messageErrors);
    displayOutput(error, 'onAddAliasCallback');
  } else {
    const response = context.messageToJson<
      jackHenry.enterprise.businessObjects.AliasAddResponse
    >();

    // must create the alias object so serialization works
    if (response && response.AliasClaimRec) {
      displayOutput(
        'Alias = ' + response.AliasClaimRec.Alias.toString(),
        'onAddAliasCallback'
      );
    } else {
      displayOutput('response is not all there', 'onAddAliasCallback');
    }

    displayOutput(
      'AliasContext = ' + response.AliasClaimRec.AliasContext.toString(),
      'onAddAliasCallback'
    );
    displayOutput(
      'AliasDesc = ' + response.AliasClaimRec.AliasDesc.toString(),
      'onAddAliasCallback'
    );
    displayOutput(
      'AliasDft = ' + response.AliasClaimRec.AliasDft.toString(),
      'onAddAliasCallback'
    );
    displayOutput(
      'AliasInstRtId = ' + response.AliasClaimRec.AliasInstRtId.toString(),
      'onAddAliasCallback'
    );
    displayOutput(
      'AppliesToApp = ' + response.AliasClaimRec.AppliesToApp.toString(),
      'onAddAliasCallback'
    );
    displayOutput(
      'UniqueId = ' + response.AliasClaimRec.UniqueId.toString(),
      'onAddAliasCallback'
    );

    displayOutput(
      'Alias add response is ' + response.RsStat,
      'onAddAliasCallback'
    );

    // NOTE: Might want to get current user to refresh the token with the new alias in it.
    refreshCurrentUser();

    displayOutput(
      'Alias was Added To Current User ' +
        this.currentUser.FirstName +
        ' ' +
        this.currentUser.LastName,
      'onAddAliasCallback'
    );
  }
}

function onAliasModCallback(context: jackHenry.xperience.MessageContext) {
  if (!context.isValid()) {
    const error = Helpers.errorsToString(context.messageErrors);
    displayOutput(error, 'onAliasCallback');
  } else {
    const response = context.messageToJson<
      jackHenry.enterprise.businessObjects.AliasModResponse
    >();

    displayOutput(
      'Alias delete response is ' + response.RsStat,
      'onAliasCallback'
    );

    // NOTE: Might want to get current user to refresh the token with the new alias in it.
    refreshCurrentUser();

    displayOutput(
      'Alias was Deleted To Current User ' +
        this.currentUser.FirstName +
        ' ' +
        this.currentUser.LastName,
      'onAliasCallback'
    );
  }
}

function onKerberosTokenCallback(context: jackHenry.xperience.MessageContext) {
  if (!context.isValid()) {
    const error = Helpers.errorsToString(context.messageErrors);
    displayOutput(error, 'onKerberosTokenCallback');
  } else {
    const response = context.messageToJson<
      jackHenry.enterprise.businessObjects.GetKerberosTokenResponse
    >();
    displayOutput(
      'Kerberos = ' + convertToText(response.KerberosToken),
      'onKerberosTokenCallback'
    );
  }
}

function siteRow(key: string, value: string) {
  const tr = document.createElement('tr');
  const td1 = document.createElement('td');
  const td2 = document.createElement('td');

  td1.style.fontWeight = '400';
  td1.style.backgroundColor = '#e6e3d9';
  td1.style.width = '15%';
  td1.style.whiteSpace = 'nowrap';
  td1.style.overflow = 'hidden';
  td1.style.textOverflow = 'ellipsis';
  td1.style.padding = '10px 7px';
  td1.style.verticalAlign = 'top';
  td2.style.borderLeft = '1px solid #777';
  td2.style.fontWeight = '500';
  td2.style.width = '85%';
  td2.style.wordBreak = 'break-all';
  td2.style.padding = '10px 7px';
  td2.style.verticalAlign = 'top';

  td1.innerText = key;
  td2.innerText = value;

  tr.appendChild(td1);
  tr.appendChild(td2);
  tr.style.borderBottom = '1px solid #777';

  return tr;
}

function stripSiteInfo(array) {
  const table = document.getElementById('site');
  const outer = array.SiteInfo;

  for (let i = 0; i < outer.length; i++) {
    const record = outer[i] as Info;
    table.appendChild(siteRow('SIName', record.SIName));
    table.appendChild(siteRow('SIType', record.SIType));
    table.appendChild(siteRow('SIDesc', record.SIDesc));
    table.appendChild(siteRow('SIVal', record.SIVal));

    const tr = document.createElement('tr');
    tr.style.borderBottom = '5px solid #777';
    table.appendChild(tr);
  }
}

function onSiteInfoCallback(context: jackHenry.xperience.MessageContext) {
  if (!context.isValid()) {
    const error = Helpers.errorsToString(context.messageErrors);
    displayOutput(error, 'onSiteInfoCallback');
  } else {
    const response = context.messageToJson<
      jackHenry.enterprise.businessObjects.SiteInfoInqResponse
    >();
    stripSiteInfo(response.SiteInfoArray);
  }
}
