/// <reference path="jackhenry.enterprise.businessObjects.js" />
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var jackHenry;
(function (jackHenry) {
    var xperience;
    (function (xperience) {
        'use strict';
        var InternalCommunication = (function () {
            function InternalCommunication() {
            }
            InternalCommunication.publish = function (message, messageOptions) {
                var receipt = jackHenry.xperience.messageBusProxy.Publish(message, JSON.stringify(messageOptions));
                var receiptBO = JSON.parse(receipt);
                return new MessageReceipt(receiptBO);
            };
            InternalCommunication.disconnect = function () {
                jackHenry.xperience.messageBusProxy.Disconnect();
            };
            InternalCommunication.connect = function (message) {
                var response = jackHenry.xperience.messageBusProxy.Connect(message);
                return response;
            };
            InternalCommunication.startIsFrameworkReadyCheck = function (timeout) {
                InternalCommunication.isFrameworkReadyCheckTimer = setInterval(function () {
                    InternalCommunication.stopIsFrameworkReadyCheck();
                    jackHenry.xperience.messageBus.frameworkReady(false);
                }, timeout);
            };
            InternalCommunication.stopIsFrameworkReadyCheck = function () {
                if (InternalCommunication.isFrameworkReadyCheckTimer) {
                    clearTimeout(InternalCommunication.isFrameworkReadyCheckTimer);
                }
            };
            InternalCommunication.xmlSerializeObject = function (itemToSerialize) {
                return jackHenry.xperience.messageBusProxy.XmlSerializeObject(itemToSerialize);
            };
            InternalCommunication.isFrameworkReadyCheckCount = 0;
            return InternalCommunication;
        }());
        var MessageContext = (function () {
            function MessageContext(messageContainer, jsonMessage) {
                this._messageContainer = messageContainer;
                this._jsonMessage = jsonMessage;
            }
            Object.defineProperty(MessageContext.prototype, "id", {
                get: function () {
                    return this._messageContainer.MsgBusHeader.MsgId;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(MessageContext.prototype, "parentId", {
                get: function () {
                    return this._messageContainer.MsgBusHeader.MsgParentId;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(MessageContext.prototype, "source", {
                get: function () {
                    return this._messageContainer.MsgBusHeader.MsgSource;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(MessageContext.prototype, "action", {
                get: function () {
                    return this._messageContainer.MsgBusHeader.MsgAction;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(MessageContext.prototype, "sourceRoute", {
                get: function () {
                    if (this._sourceRoute == null) {
                        this._sourceRoute = new Route();
                        if (this._messageContainer.MsgBusHeader.MsgSrcRt != null) {
                            this._sourceRoute.sessionId = this._messageContainer.MsgBusHeader.MsgSrcRt.ClientId;
                            this._sourceRoute.user = new UserRoute();
                            if (this._messageContainer.MsgBusHeader.MsgSrcRt.Usr != null) {
                                this._sourceRoute.user.userName = this._messageContainer.MsgBusHeader.MsgSrcRt.Usr.UsrId;
                            }
                        }
                    }
                    return this._sourceRoute;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(MessageContext.prototype, "messageErrors", {
                get: function () {
                    if (this._messageContainer.MsgBusHeader.MsgError == null) {
                        return null;
                    }
                    return this._messageContainer.MsgBusHeader.MsgError.MsgError;
                },
                enumerable: true,
                configurable: true
            });
            MessageContext.prototype.isValid = function () {
                if (this.messageErrors != null && this.messageErrors.length > 0) {
                    return false;
                }
                return true;
            };
            MessageContext.prototype.messageToJson = function () {
                if (this._jsonMessage == null || this._jsonMessage[this.action] == null) {
                    return null;
                }
                return this._jsonMessage[this.action];
            };
            MessageContext.prototype.messageToXmlString = function () {
                if (this._messageContainer == null || this._messageContainer.MsgBusBody == null) {
                    return null;
                }
                return this._messageContainer.MsgBusBody.MsgPayload;
            };
            return MessageContext;
        }());
        xperience.MessageContext = MessageContext;
        var SubscribedMessageContext = (function (_super) {
            __extends(SubscribedMessageContext, _super);
            function SubscribedMessageContext(messageContainer, jsonMessage, messageBus) {
                _super.call(this, messageContainer, jsonMessage);
                this._subscriberMessageContainer = messageContainer;
                this._messageBus = messageBus;
            }
            SubscribedMessageContext.prototype.reply = function (message, errors) {
                if (errors === void 0) { errors = null; }
                var options = new MessageOptions();
                options.destinationIds.push(this._subscriberMessageContainer.MsgBusHeader.MsgSource);
                options.parentId = this._subscriberMessageContainer.MsgBusHeader.MsgId;
                if (this._subscriberMessageContainer.MsgBusHeader.MsgSrcRt != null) {
                    var sourceRoute = new Route();
                    sourceRoute.sessionId = this._subscriberMessageContainer.MsgBusHeader.MsgSrcRt.ClientId,
                        sourceRoute.user = new UserRoute();
                    if (this._subscriberMessageContainer.MsgBusHeader.MsgSrcRt.Usr != null) {
                        sourceRoute.user.userName = this._subscriberMessageContainer.MsgBusHeader.MsgSrcRt.Usr.UsrId;
                    }
                    options.destinationRoutes.push(sourceRoute);
                }
                options.errors = errors;
                this._messageBus.publish(message, options);
            };
            return SubscribedMessageContext;
        }(MessageContext));
        xperience.SubscribedMessageContext = SubscribedMessageContext;
        var MessageReceipt = (function () {
            function MessageReceipt(receiptFromMb) {
                this.receipt = receiptFromMb;
            }
            Object.defineProperty(MessageReceipt.prototype, "messageId", {
                get: function () {
                    return this.receipt.MsgId;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(MessageReceipt.prototype, "errors", {
                get: function () {
                    if (this.receipt.MsgError == null) {
                        return null;
                    }
                    return this.receipt.MsgError.MsgError;
                },
                enumerable: true,
                configurable: true
            });
            MessageReceipt.prototype.isValid = function () {
                if (this.errors != null && this.errors.length > 0) {
                    return false;
                }
                return true;
            };
            return MessageReceipt;
        }());
        xperience.MessageReceipt = MessageReceipt;
        // TODO: put interface on this and only expose registerResponseHandler
        var ResponseCallback = (function () {
            function ResponseCallback() {
            }
            ResponseCallback.prototype.registerResponseHandler = function (handler) {
                if (this.responseHandler != null) {
                    throw "A response handler has already been registered";
                }
                if (this.receipt != null && !this.receipt.isValid()) {
                    this.fillContextWithReceiptError();
                }
                this.responseHandler = handler;
                if (this.context != null) {
                    this.responseHandler(this.context);
                }
                return this;
            };
            ResponseCallback.prototype.handleResponse = function (messageContext) {
                this.context = messageContext;
                if (this.receipt != null && !this.receipt.isValid()) {
                    this.fillContextWithReceiptError();
                }
                if (this.responseHandler != null) {
                    this.responseHandler(messageContext);
                }
            };
            ResponseCallback.prototype.fillContextWithReceiptError = function () {
                var errors = new Array();
                var error = new jackHenry.enterprise.businessObjects.MsgRec_CType();
                error.ErrCat = "Error",
                    error.ErrDesc = "The message receipt had errors therefore a response can not be received.",
                    error.ErrCode = "4004";
                errors.push(error);
                for (var i = 0; i < this.receipt.errors.length; i++) {
                    errors.push(this.receipt.errors[i]);
                }
                if (this.context == null) {
                    var message = new jackHenry.enterprise.businessObjects.MsgBusMsg_CType();
                    message.MsgBusHeader = new jackHenry.enterprise.businessObjects.MsgBusHeader_CType();
                    message.MsgBusHeader.MsgId = this.receipt.messageId;
                    message.MsgBusHeader.MsgError = new jackHenry.enterprise.businessObjects.MsgErrorType();
                    message.MsgBusHeader.MsgError.MsgError = errors;
                    this.context = new MessageContext(message, null);
                }
                else {
                    for (var i = 0; i < this.receipt.errors.length; i++) {
                        this.context.messageErrors.push(this.receipt.errors[i]);
                    }
                }
            };
            return ResponseCallback;
        }());
        xperience.ResponseCallback = ResponseCallback;
        var MessageSubscription = (function () {
            function MessageSubscription(messageAction) {
                this.action = messageAction;
                this.handlers = [];
            }
            MessageSubscription.prototype.addHandler = function (messageHandler) {
                this.handlers.push(messageHandler);
            };
            return MessageSubscription;
        }());
        var Guid = (function () {
            function Guid() {
            }
            Guid.newGuid = function () {
                return this.s4() + this.s4() + '-' + this.s4() + '-' + this.s4() + '-' + this.s4() + '-' + this.s4() + this.s4() + this.s4();
            };
            Guid.s4 = function () {
                return Math.floor((1 + Math.random()) * 0x10000)
                    .toString(16)
                    .substring(1);
            };
            return Guid;
        }());
        xperience.Guid = Guid;
        var MessageSubscriptionToken = (function () {
            function MessageSubscriptionToken() {
                this.token = Guid.newGuid();
            }
            return MessageSubscriptionToken;
        }());
        xperience.MessageSubscriptionToken = MessageSubscriptionToken;
        var MessageSubscriptionHandler = (function () {
            function MessageSubscriptionHandler() {
                this.subscriptionToken = new MessageSubscriptionToken();
            }
            return MessageSubscriptionHandler;
        }());
        xperience.MessageSubscriptionHandler = MessageSubscriptionHandler;
        var UserRoute = (function () {
            function UserRoute() {
            }
            return UserRoute;
        }());
        xperience.UserRoute = UserRoute;
        var Route = (function () {
            function Route() {
            }
            return Route;
        }());
        xperience.Route = Route;
        var MessageOptions = (function () {
            function MessageOptions() {
                this.messageContentType = "Json";
                this.destinationIds = new Array();
                this.destinationRoutes = new Array();
                this.errors = new Array();
                this.requestReceipt = false;
                this.forceAct = false;
            }
            return MessageOptions;
        }());
        xperience.MessageOptions = MessageOptions;
        var MessageBus = (function () {
            function MessageBus(messageBusSubscriberService) {
                this._onReadyCallback = null;
                this._isFrameworkReady = false;
                this.messageBusSubscriberService = messageBusSubscriberService;
            }
            Object.defineProperty(MessageBus.prototype, "clientId", {
                get: function () {
                    return this._clientId;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(MessageBus.prototype, "sessionId", {
                get: function () {
                    return this._sessionId;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(MessageBus.prototype, "isConnected", {
                get: function () {
                    return this.messageBusSubscriberService.isConnected();
                },
                enumerable: true,
                configurable: true
            });
            MessageBus.prototype.connect = function () {
                return this.messageBusSubscriberService.connect();
            };
            MessageBus.prototype.disconnect = function (clearSubscriptions) {
                this.messageBusSubscriberService.disconnect(clearSubscriptions);
            };
            MessageBus.prototype.publish = function (message, messageOptions) {
                if (messageOptions === void 0) { messageOptions = new MessageOptions(); }
                var serializedMessage;
                if (message.ToJsonString != null) {
                    serializedMessage = message.ToJsonString();
                }
                else if (typeof message == 'string') {
                    // assume it is an xml string
                    serializedMessage = message;
                }
                return InternalCommunication.publish(serializedMessage, messageOptions);
            };
            MessageBus.prototype.send = function (message, messageOptions) {
                if (messageOptions === void 0) { messageOptions = new MessageOptions(); }
                if (messageOptions.destinationIds.length > 1) {
                    throw "Send can not handle multiple destinations";
                }
                var callback = new ResponseCallback();
                this.messageBusSubscriberService.waitingForResponse.push(callback);
                callback.receipt = this.publish(message, messageOptions);
                callback.messageId = callback.receipt.messageId;
                return callback;
            };
            MessageBus.prototype.subscribe = function (messageAction, handler) {
                return this.messageBusSubscriberService.subscribe(messageAction, handler);
            };
            MessageBus.prototype.unsubscribe = function (subscriptionToken) {
                return this.messageBusSubscriberService.unsubscribe(subscriptionToken);
            };
            MessageBus.prototype.onReady = function (onReadyCallback, timeout) {
                if (timeout === void 0) { timeout = 5000; }
                this._onReadyCallback = onReadyCallback;
                if (this._isFrameworkReady) {
                    if (this._onReadyCallback != null) {
                        this._onReadyCallback();
                    }
                }
                else {
                    InternalCommunication.startIsFrameworkReadyCheck(timeout);
                }
            };
            MessageBus.prototype.frameworkReady = function (isReady) {
                if (isReady === void 0) { isReady = true; }
                InternalCommunication.stopIsFrameworkReadyCheck();
                if (!this._isFrameworkReady) {
                    this._isFrameworkReady = isReady;
                    if (this._onReadyCallback != null) {
                        this._onReadyCallback();
                    }
                }
            };
            MessageBus.prototype.xmlSerializeObject = function (itemToSerialize) {
                var serializedMessage;
                if (itemToSerialize.ToJsonString != null) {
                    serializedMessage = itemToSerialize.ToJsonString();
                }
                else if (typeof itemToSerialize == 'string') {
                    // assume it is an xml string
                    serializedMessage = itemToSerialize;
                }
                return InternalCommunication.xmlSerializeObject(serializedMessage);
            };
            return MessageBus;
        }());
        xperience.MessageBus = MessageBus;
        var MessageBusSubscriberService = (function () {
            function MessageBusSubscriberService() {
                this.messageSubscriptions = new Array();
                this._isConnected = false;
                this.waitingForResponse = new Array();
            }
            MessageBusSubscriberService.prototype.disconnect = function (clearSubscriptions) {
                if (this._isConnected) {
                    var disconnectMessage = new jackHenry.enterprise.businessObjects.MsgBusDisconnect_CType();
                    disconnectMessage.ClientId = this.clientId;
                    InternalCommunication.disconnect();
                    this.waitingForResponse = new Array();
                    this.messageSubscriptions = new Array();
                    this._isConnected = false;
                    this.connectionResponseResult = null;
                    this.clientId = null;
                    jackHenry.xperience.isFrameworkHosted = false;
                    jackHenry.xperience.frameworkVersion = "";
                }
            };
            MessageBusSubscriberService.prototype.connect = function () {
                if (this._isConnected) {
                    return this.connectionResponseResult;
                }
                else {
                    var request = new jackHenry.enterprise.businessObjects.MsgBusConnect_CType();
                    request.ClientId = this.clientId;
                    try {
                        this.connectionResponseResult = InternalCommunication.connect(request);
                    }
                    catch (e) {
                        jackHenry.xperience.isFrameworkHosted = false;
                        jackHenry.xperience.frameworkVersion = "";
                        var error = new jackHenry.enterprise.businessObjects.MsgRec_CType();
                        error.ErrCat = "Error";
                        error.ErrDesc = e;
                        this.connectionResponseResult = new jackHenry.enterprise.businessObjects.MsgBusConnectRs_CType();
                        this.connectionResponseResult.MsgError = new jackHenry.enterprise.businessObjects.MsgErrorType();
                        this.connectionResponseResult.MsgError.MsgError = new Array();
                        this.connectionResponseResult.MsgError.MsgError.push(error);
                    }
                    if (this.connectionResponseResult != null &&
                        this.connectionResponseResult.MsgError != null &&
                        this.connectionResponseResult.MsgError.MsgError != null &&
                        this.connectionResponseResult.MsgError.MsgError.length > 0) {
                        this._isConnected = false;
                        jackHenry.xperience.isFrameworkHosted = false;
                        jackHenry.xperience.frameworkVersion = "";
                        return this.connectionResponseResult;
                    }
                    else {
                        if (this.connectionResponseResult != null) {
                            this.clientId = this.connectionResponseResult.ClientId;
                        }
                        this._isConnected = true;
                        jackHenry.xperience.isFrameworkHosted = true;
                        return this.connectionResponseResult;
                    }
                }
            };
            MessageBusSubscriberService.prototype.subscribe = function (messageAction, handler) {
                var subscription;
                if (this.messageSubscriptions.contains(messageAction)) {
                    subscription = this.messageSubscriptions[messageAction];
                }
                else {
                    subscription = new MessageSubscription(messageAction);
                    this.messageSubscriptions[messageAction] = subscription;
                }
                var subscriptionHandler = new MessageSubscriptionHandler();
                subscriptionHandler.handler = handler;
                subscription.addHandler(subscriptionHandler);
                return subscriptionHandler.subscriptionToken;
            };
            MessageBusSubscriberService.prototype.unsubscribe = function (subscriptionToken) {
                for (var i = 0; i < this.messageSubscriptions.length; i++) {
                    var handlers = this.messageSubscriptions[i].handlers;
                    for (var j = 0; j < handlers.length; j++) {
                        if (handlers[j].subscriptionToken.token == subscriptionToken.token) {
                            handlers.splice(j, 1);
                            return true;
                        }
                    }
                }
                return false;
            };
            MessageBusSubscriberService.prototype.receiveMessage = function (jsonMessage, message, messageContainer) {
                if (messageContainer === void 0) { messageContainer = new jackHenry.enterprise.businessObjects.MsgBusMsg_CType(); }
                if ((!message || message == null) && (!messageContainer || messageContainer == null)) {
                    return;
                }
                if (messageContainer.MsgBusHeader == null) {
                    messageContainer.MsgBusHeader = new jackHenry.enterprise.businessObjects.MsgBusHeader_CType();
                }
                else {
                    // NOTE:  The Xperience contract for the MsgBusHeader doesn't match the actual XSD contract, so we need to do a conversion:
                    var hdrToString = JSON.stringify(messageContainer.MsgBusHeader);
                    // Now I can work with it as an 'any' object.
                    var hdrFromString = JSON.parse(hdrToString);
                    // Convert the MsgError array items:
                    if (hdrFromString.MsgError && hdrFromString.MsgError.length > 0) {
                        messageContainer.MsgBusHeader.MsgError = new jackHenry.enterprise.businessObjects.MsgErrorType();
                        messageContainer.MsgBusHeader.MsgError.MsgError = new Array();
                        for (var i = 0; i < hdrFromString.MsgError.length; i++) {
                            var err = hdrFromString.MsgError[i];
                            messageContainer.MsgBusHeader.MsgError.MsgError.push(err);
                        }
                    }
                    // Convert the MsgDests array items:
                    if (hdrFromString.MsgDests && hdrFromString.MsgDests.length > 0) {
                        messageContainer.MsgBusHeader.MsgDests = new jackHenry.enterprise.businessObjects.MsgDestsType();
                        messageContainer.MsgBusHeader.MsgDests.MsgDest = new Array();
                        for (var i = 0; i < hdrFromString.MsgDests.length; i++) {
                            var dest = hdrFromString.MsgDests[i];
                            messageContainer.MsgBusHeader.MsgDests.MsgDest.push(dest);
                        }
                    }
                }
                // NOTE: had to break apart message and container due to deserialization issues with JSON and an XML property
                if (messageContainer.MsgBusBody == null || messageContainer.MsgBusBody.MsgPayload == null) {
                    messageContainer.MsgBusBody = new jackHenry.enterprise.businessObjects.MsgBusBody_CType();
                    messageContainer.MsgBusBody.MsgPayload = message;
                }
                //NOTE: check first for handlers waiting for a specific response
                for (var i = 0; i < this.waitingForResponse.length; i++) {
                    var callback = this.waitingForResponse[i];
                    if (callback.messageId == messageContainer.MsgBusHeader.MsgParentId) {
                        var context = new MessageContext(messageContainer, jsonMessage);
                        callback.handleResponse(context);
                    }
                    ;
                }
                if (messageContainer.MsgBusBody.MsgPayload != null && messageContainer.MsgBusBody.MsgPayload != '') {
                    var handlers = this.getHandlers(messageContainer.MsgBusHeader.MsgAction);
                    if (handlers != null && handlers.length > 0) {
                        var subContext = new SubscribedMessageContext(messageContainer, jsonMessage, jackHenry.xperience.messageBus);
                        for (var i = 0; i < handlers.length; i++) {
                            handlers[i].handler(subContext);
                        }
                    }
                }
            };
            MessageBusSubscriberService.prototype.isConnected = function () {
                return this._isConnected;
            };
            MessageBusSubscriberService.prototype.getHandlers = function (messageAction) {
                var subscription = this.messageSubscriptions[messageAction];
                if (subscription == null) {
                    return [];
                }
                return subscription.handlers;
            };
            return MessageBusSubscriberService;
        }());
        xperience.MessageBusSubscriberService = MessageBusSubscriberService;
        var XpArgs = (function () {
            function XpArgs() {
                this._xmlns = "http://jackHenry.com/jxchange/JES/2008";
            }
            return XpArgs;
        }());
        xperience.XpArgs = XpArgs;
        xperience.messageBusSubscriber = new MessageBusSubscriberService();
        xperience.messageBus = new MessageBus(xperience.messageBusSubscriber);
        xperience.isFrameworkHosted = false;
        xperience.frameworkVersion = "";
        function isUpdatedVersion() {
            var oldestUpdatedVersion = "2016.2.0.0";
            if (typeof xperience.frameworkVersion === 'undefined' || xperience.frameworkVersion.length === 0) {
                return false;
            }
            var currentQuadrants = xperience.frameworkVersion.split('.');
            var oldestQuadrants = oldestUpdatedVersion.split('.');
            for (var i = 0; i < currentQuadrants.length; i++) {
                if (oldestQuadrants.length === i) {
                    return true;
                }
                if (!isNaN(Number(currentQuadrants[i])) && !isNaN(Number(oldestQuadrants[i]))) {
                    if (currentQuadrants[i] === oldestQuadrants[i]) {
                        continue;
                    }
                    else if (Number(currentQuadrants[i]) > Number(oldestQuadrants[i])) {
                        return true;
                    }
                    else {
                        return false;
                    }
                }
                else {
                    return false;
                }
            }
            if (currentQuadrants.length !== oldestQuadrants.length) {
                return false;
            }
            return true;
        }
        xperience.isUpdatedVersion = isUpdatedVersion;
        if (navigator.userAgent.indexOf('Xperience/Awesomium') > -1) {
            xperience.isFrameworkHosted = true;
        }
        else {
            xperience.isFrameworkHosted = false;
        }
        ;
    })(xperience = jackHenry.xperience || (jackHenry.xperience = {}));
})(jackHenry || (jackHenry = {}));
Array.prototype.contains = function (obj) {
    var i = this.length;
    while (i--) {
        if (this[i] === obj) {
            return true;
        }
    }
    return false;
};
//# sourceMappingURL=jackhenry.xperience.js.map
