/// <reference path="jackhenry.enterprise.businessObjects.d.ts" />
declare module jackHenry.xperience {
    class MessageContext {
        private _sourceRoute;
        private _messageContainer;
        private _jsonMessage;
        constructor(messageContainer: jackHenry.enterprise.businessObjects.MsgBusMsg_CType, jsonMessage: string);
        id: string;
        parentId: string;
        source: string;
        action: string;
        sourceRoute: Route;
        messageErrors: Array<jackHenry.enterprise.businessObjects.MsgRec_CType>;
        isValid(): boolean;
        messageToJson<T>(): T;
        messageToXmlString(): string;
    }
    class SubscribedMessageContext extends MessageContext {
        private _messageBus;
        private _subscriberMessageContainer;
        constructor(messageContainer: jackHenry.enterprise.businessObjects.MsgBusMsg_CType, jsonMessage: string, messageBus: MessageBus);
        reply(message: any): any;
        reply(message: any, errors: Array<jackHenry.enterprise.businessObjects.MsgRec_CType>): any;
    }
    class MessageReceipt {
        receipt: jackHenry.enterprise.businessObjects.MsgBusMsgRs_CType;
        constructor(receiptFromMb: jackHenry.enterprise.businessObjects.MsgBusMsgRs_CType);
        messageId: string;
        errors: Array<jackHenry.enterprise.businessObjects.MsgRec_CType>;
        isValid(): boolean;
    }
    class ResponseCallback {
        responseHandler: {
            (messageContext: MessageContext): void;
        };
        context: MessageContext;
        receipt: MessageReceipt;
        messageId: string;
        registerResponseHandler(handler: {
            (messageContext: MessageContext): void;
        }): ResponseCallback;
        handleResponse(messageContext: MessageContext): void;
        private fillContextWithReceiptError();
    }
    class Guid {
        static newGuid(): string;
        private static s4();
    }
    class MessageSubscriptionToken {
        token: string;
        constructor();
    }
    class MessageSubscriptionHandler {
        handler: {
            (context: MessageContext): void;
        };
        subscriptionToken: MessageSubscriptionToken;
        constructor();
    }
    class UserRoute {
        userName: string;
    }
    class Route {
        sessionId: string;
        user: UserRoute;
    }
    class MessageOptions {
        requestReceipt: boolean;
        parentId: string;
        destinationIds: Array<string>;
        destinationRoutes: Array<Route>;
        errors: Array<jackHenry.enterprise.businessObjects.MsgRec_CType>;
        forceAct: boolean;
        messageContentType: string;
        constructor();
    }
    class MessageBus {
        private messageBusSubscriberService;
        private _clientId;
        private _sessionId;
        private _onReadyCallback;
        private _isFrameworkReady;
        constructor(messageBusSubscriberService: MessageBusSubscriberService);
        clientId: string;
        sessionId: string;
        isConnected: boolean;
        connect(): jackHenry.enterprise.businessObjects.MsgBusConnectRs_CType;
        disconnect(clearSubscriptions: boolean): void;
        publish(message: any): MessageReceipt;
        publish(message: any, messageOptions: MessageOptions): MessageReceipt;
        send(message: any): ResponseCallback;
        send(message: any, messageOptions: MessageOptions): ResponseCallback;
        subscribe(messageAction: string, handler: {
            (rs: SubscribedMessageContext): void;
        }): MessageSubscriptionToken;
        unsubscribe(subscriptionToken: MessageSubscriptionToken): boolean;
        onReady(onReadyCallback: {
            (): void;
        }, timeout?: number): void;
        frameworkReady(isReady?: boolean): void;
        xmlSerializeObject(itemToSerialize: jackHenry.enterprise.businessObjects.BusinessObject): string;
    }
    class MessageBusSubscriberService {
        private messageSubscriptions;
        private _isConnected;
        waitingForResponse: Array<ResponseCallback>;
        clientId: string;
        private connectionResponseResult;
        disconnect(clearSubscriptions: boolean): void;
        connect(): jackHenry.enterprise.businessObjects.MsgBusConnectRs_CType;
        subscribe(messageAction: string, handler: {
            (rs: SubscribedMessageContext): void;
        }): MessageSubscriptionToken;
        unsubscribe(subscriptionToken: MessageSubscriptionToken): boolean;
        receiveMessage(jsonMessage: string, message: string, messageContainer?: jackHenry.enterprise.businessObjects.MsgBusMsg_CType): void;
        isConnected(): boolean;
        private getHandlers(messageAction);
    }
    class XpArgs {
        MessageBusSessionId: string;
        ClientId: string;
        Theme: string;
        ThemeType: string;
        ColorType: string;
        ThemeModeType: string;
        StartUpUri: string;
        TraceEnabled: boolean;
        HelpPopupState: string;
        AnimationState: string;
        ZoomState: number;
        AuthenMode: string;
        NotHostedInXpTab: boolean;
        UseXmlSerializerEndpoint: boolean;
        private _xmlns;
        constructor();
    }
    interface MessageBusProxy {
        Publish(message: string, messageOptions: string): string;
        Connect(message: jackHenry.enterprise.businessObjects.MsgBusConnect_CType): jackHenry.enterprise.businessObjects.MsgBusConnectRs_CType;
        Disconnect(): void;
        XmlSerializeObject(itemToSerialize: string): string;
    }
    var messageBusSubscriber: MessageBusSubscriberService;
    var messageBus: MessageBus;
    var isFrameworkHosted: boolean;
    var messageBusProxy: MessageBusProxy;
    var startupArguments: XpArgs;
    var frameworkVersion: string;
    function isUpdatedVersion(): boolean;
}
interface Array<T> {
    contains(obj: any): boolean;
}
