import { Injectable } from '@angular/core';

@Injectable()
export class Helpers {
  public currentUser = null;

  static errorsToString(
    errors: Array<jackHenry.enterprise.businessObjects.MsgRec_CType>
  ): string {
    let result = '';
    if (errors != null && errors.length > 0) {
      for (let i = 0; i < errors.length; i++) {
        const error = Helpers.errorRecToString(errors[i]);
        result += error;
      }
    }

    return result;
  }

  static callbackErrors(callback): string {
    if (callback && callback.receipt && callback.receipt.errors) {
      const errors = Helpers.collectionErrors(callback.receipt.errors);
      return errors;
    }
  }

  static collectionErrors(
    errors: Array<jackHenry.enterprise.businessObjects.MsgRec_CType>
  ) {
    let comment: string;
    if (errors && errors.length > 0) {
      errors.forEach(element => {
        const error = Helpers.messageError(element);
        comment += error;
      });
    }

    return comment;
  }

  static messageError(
    record: jackHenry.enterprise.businessObjects.MsgRec_CType
  ): string {
    const error =
      'Error /n  ----> ErrCat.Value:  ' +
      record.ErrCat +
      '/n  ----> ErrCode.Value:  ' +
      record.ErrCode +
      '/n  ----> ErrDesc.Value:  ' +
      record.ErrDesc +
      '/n  ----> ErrElem.Value:  ' +
      record.ErrElem +
      '/n  ----> ErrElemVal.Value:  ' +
      record.ErrElemVal +
      '/n  ----> ErrLoc.Value:  ' +
      record.ErrLoc;
    console.log(error);

    return error;
  }

  static errorRecToString(
    record: jackHenry.enterprise.businessObjects.MsgRec_CType
  ): string {
    const error =
      'Error \n' +
      'Cat:  ' +
      record.ErrCat +
      '\nCode: ' +
      record.ErrCode +
      '\nDescription: ' +
      record.ErrDesc +
      '\nElement: ' +
      record.ErrElem +
      '\nValue: :' +
      record.ErrElemVal +
      '\nLocation: ' +
      record.ErrLoc +
      '\n';

    return error;
  }

  static fillHeader(request: any): void {
    request.JESMsgRqHdr = Helpers.jesMsgRqHeader();
  }

  static replaceAll(find: string, replace: string, str: string): string {
    if (
      find &&
      find.length > 0 &&
      replace &&
      replace.length > 0 &&
      str &&
      str.length > 0
    ) {
      return str.replace(new RegExp(find, 'g'), replace);
    } else {
      return find;
    }
  }

  // When sending string data to a Web API, or message bus message,
  // all single quotes from JSON must be converted to double quotes:
  static fixQuotes(str: string): string {
    if (str && str.length > 0) {
      // tslint:disable-next-line:quotemark
      return Helpers.replaceAll("'", '"', str);
    } else {
      return str;
    }
  }

  static guid() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
      // tslint:disable-next-line:no-bitwise
      const r = (Math.random() * 16) | 0,
        // tslint:disable-next-line:no-bitwise
        v = c === 'x' ? r : (r & 0x3) | 0x8;
      return v.toString(16);
    });
  }

  private static jesMsgRqHeader(): jackHenry.enterprise.businessObjects.JESMsgRqHdr_CType {
    const header = new jackHenry.enterprise.businessObjects.JESMsgRqHdr_CType();
    header.JESHdr = Helpers.jesHeader();
    return header;
  }

  private static jesHeader(): jackHenry.enterprise.businessObjects.JESHdr_CType {
    const header = new jackHenry.enterprise.businessObjects.JESHdr_CType();
    // header.ConsumerProd = 'XpAngular';
    header.jXLogTrackingId = Helpers.guid();
    header.ValidConsmName = 'JHA';
    header.ValidConsmProd = 'JHA';
    return header;
  }
}
